I messed up the IRQ state when jumping off the fast path due to
invocation of a ptregs-using syscall.  This bug shouldn't have had
any impact yet, but it would have caused problems with subsequent
context tracking cleanups.

Fixes: 1e423bff959e x86/entry/64: ("Migrate the 64-bit syscall slow path to C")
Signed-off-by: Andy Lutomirski <l...@kernel.org>
---
 arch/x86/entry/entry_64.S | 19 +++++++++++++------
 1 file changed, 13 insertions(+), 6 deletions(-)

diff --git a/arch/x86/entry/entry_64.S b/arch/x86/entry/entry_64.S
index 567aa522ac0a..9f7bb808035e 100644
--- a/arch/x86/entry/entry_64.S
+++ b/arch/x86/entry/entry_64.S
@@ -191,8 +191,8 @@ entry_SYSCALL_64_fastpath:
 
        /*
         * This call instruction is handled specially in stub_ptregs_64.
-        * It might end up jumping to the slow path.  If it jumps, RAX is
-        * clobbered.
+        * It might end up jumping to the slow path.  If it jumps, RAX
+        * and all argument registers are clobbered.
         */
        call    *sys_call_table(, %rax, 8)
 .Lentry_SYSCALL_64_after_fastpath_call:
@@ -315,17 +315,24 @@ END(entry_SYSCALL_64)
 ENTRY(stub_ptregs_64)
        /*
         * Syscalls marked as needing ptregs land here.
-        * If we are on the fast path, we need to save the extra regs.
-        * If we are on the slow path, the extra regs are already saved.
+        * If we are on the fast path, we need to save the extra regs,
+        * which we achieve by trying again on the slow path.  If we are on
+        * the slow path, the extra regs are already saved.
         *
         * RAX stores a pointer to the C function implementing the syscall.
+        * IRQs are on.
         */
        cmpq    $.Lentry_SYSCALL_64_after_fastpath_call, (%rsp)
        jne     1f
 
-       /* Called from fast path -- pop return address and jump to slow path */
+       /*
+        * Called from fast path -- disable IRQs again, pop return address
+        * and jump to slow path
+        */
+       DISABLE_INTERRUPTS(CLBR_NONE)
+       TRACE_IRQS_OFF
        popq    %rax
-       jmp     entry_SYSCALL64_slow_path       /* called from fast path */
+       jmp     entry_SYSCALL64_slow_path
 
 1:
        /* Called from C */
-- 
2.5.0

Reply via email to